import 'dart:async';
import 'package:flutter/material.dart';

class ContactListFlutter extends StatefulWidget {
  final IndexedWidgetBuilder itemBuilder;
  final double itemHeight;
  final double headerItemHeight;
  final List<Widget> headerWidgets;
  final List<String> items;
  final TextStyle selectedStyle;
  final TextStyle unSelectedStyle;

  const ContactListFlutter({
    Key key,
    @required this.itemBuilder,
    @required this.items,
    @required this.itemHeight,
    this.headerItemHeight,
    this.headerWidgets,
    this.selectedStyle,
    this.unSelectedStyle,
  }) : super(key: key);

  @override
  _ContactListWidgetState createState() => _ContactListWidgetState();
}

class _ContactListWidgetState extends State<ContactListFlutter> {
  String _alphabetChar = "";
  String selectedAlphabet = "";
  ScrollController _scrollController;
  Map<String, int> _alphabetIndexRelationMap = Map();
  Map<double, String> _position = Map();
  List<String> _alphabets = List();
  var alphabetContainerKey = GlobalKey();
  final _pixelUpdates = StreamController<double>();

  @override
  void initState() {
    _initScrollCallback();
    _scrollController = ScrollController(keepScrollOffset: false);
    _scrollController.addListener(() {
      _listener();
    });
    for (int index = 0; index < widget.items.length; index++) {
      _alphabetLogic(widget.items[index], index);
    }
    _alphabets.addAll(_alphabetIndexRelationMap.keys);
    selectedAlphabet = _position[0.0];
    super.initState();
  }

  _listener() {
    _pixelUpdates.add(_scrollController.position.pixels);
  }

  _initScrollCallback() {
    _pixelUpdates.stream.listen((pixels) {
      var childLength = widget.items.length;
      double maxScrollExtent = _scrollController.position.maxScrollExtent > 0
          ? _scrollController.position.maxScrollExtent
          : 1;
      var tempSelectedIndex =
          ((pixels / maxScrollExtent) * childLength).toInt();
      if (tempSelectedIndex >= childLength) {
        tempSelectedIndex = childLength - 1;
      }
      String mapKey;
      if (tempSelectedIndex < 0 || tempSelectedIndex >= widget.items.length) {
        return;
      }
      if (widget.items[tempSelectedIndex].contains(RegExp("^x\\d\$"))) {
        mapKey = widget.items[tempSelectedIndex].isNotEmpty
            ? (widget.items[tempSelectedIndex].toUpperCase())
            : "#";
      } else {
        mapKey = widget.items[tempSelectedIndex].isNotEmpty
            ? (widget.items[tempSelectedIndex].toUpperCase())
            : "#";
      }

      if (selectedAlphabet != mapKey) {
        setState(() {
          selectedAlphabet = mapKey;
        });
      }
    });
  }

  String _alphabetLogic(String displayName, int index) {
    if (displayName.isEmpty) displayName = "#";
    if (_alphabetChar.toUpperCase() !=
        displayName.trim().substring(0, 1).toUpperCase()) {
      _alphabetChar = displayName.trim().substring(0, 1).toUpperCase();
      _alphabetIndexRelationMap[_alphabetChar] = index;
      _position[index * widget.itemHeight] = _alphabetChar;
      return _alphabetChar;
    } else
      return "";
  }

  @override
  void dispose() {
    _pixelUpdates.close();
    _scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[_itemsList(context)],
    );
  }

  Widget _itemsList(BuildContext context) {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Expanded(
          child: Container(
            child: CustomScrollView(
              controller: _scrollController,
              shrinkWrap: true,
              slivers: [
                SliverList(
                  delegate: SliverChildListDelegate(
                      widget.headerWidgets ?? [Container()]),
                ),
                SliverFixedExtentList(
                  delegate: SliverChildBuilderDelegate(
                    widget.itemBuilder,
                    childCount: widget.items.length,
                  ),
                  itemExtent: widget.itemHeight,
                ),
              ],
            ),
          ),
        ),
        _alphabetsListWidget()
      ],
    );
  }

  _alphabetsListWidget() {
    return SingleChildScrollView(
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: List.generate(
          _alphabets.length,
          (index) => InkWell(
            onTap: () {
              var jumpIndex = _alphabetIndexRelationMap[_alphabets[index]];
              var jumpAt;
              if (widget.headerWidgets != null &&
                  widget.headerItemHeight != null) {
                jumpAt = (jumpIndex * widget.itemHeight) +
                    (widget.headerItemHeight * widget.headerWidgets.length);
              } else {
                jumpAt = (jumpIndex * widget.itemHeight);
              }
              selectedAlphabet = _alphabets[index];
              _scrollController.jumpTo(jumpAt);
              setState(() {});
            },
            child: Container(
              width: 30,
              alignment: Alignment.center,
              padding: EdgeInsets.symmetric(
                  vertical: 0.5 * MediaQuery.of(context).devicePixelRatio),
              child: Text(
                _alphabets[index],
                style: _alphabets[index] == selectedAlphabet
                    ? widget.selectedStyle ??
                        TextStyle(
                            color: Colors.black,
                            fontWeight: FontWeight.w800,
                            fontSize: 15)
                    : widget.unSelectedStyle ??
                        TextStyle(
                            color: Colors.black54,
                            fontWeight: FontWeight.w800,
                            fontSize: 15),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
